%% This code reproduces Figure A2

% Supports Sign, Zero and Narrative Sign Restrictions, as well as Conjugate
% Priors on the Reduced Form
% Developed by Antolín-Díaz and Rubio-Ramírez (2018): Narrative Sign Restrictions for SVARs. American Economic Review

clear
clc

addpath('data')
addpath('functions')


%% Reduced form Settings 

% Data Location and Adjustments

datafile = 'dataSVAR';
panelselect = [1:3];                % Choose subset and/or re-order variables (optional); Empty = use all;

startYear = 1955;                       % Choose start year
endYear = 2007;                           

% Model Specification

constant = 0;                           % Add constant in VAR
exog = [];                              % Specify Position of Exogenous Variables (optional);

p = 3;                                  % Maximum lag order of factor VAR
    
h = 0;                                  % Desired forecast horizon


%% Reduced Form Priors

prior_settings.prior_family = 'conjugate';
prior_settings.prior = 'Minnesota';                % Select 'flat' or 'Minnesota'


% Additional Settings of the Minnesota Prior
prior_settings.stationary = [];             % Select which variables are a priori stationary

prior_settings.hyperparameters.lambda = .1; % Tightness of the Minnesota Prior; 
prior_settings.hyperparameters.alpha = 2;   % Decay of the Minnesota Prior; 
% prior_settings.hyperparameters.psi = psi;   % Scale of the Minnesota Prior; 
prior_settings.LongRunPrior = 0;
prior_settings.LongRunPrior2 = 0;
prior_settings.NoCointegration = 0;         % Also known as Sum of Coefficients (1=Yes,0=no)
prior_settings.SingleUnitRoot = 0;          % Also  known as Dummy Initial Observation (1=Yes,0=no)

prior_settings.hyperparameters.mu = 0;      % Tightness of the No Cointegration / Sum of Coefficients Prior
prior_settings.hyperparameters.theta = 0;   % Tightness of the Dummy Initial Observation Priors

%% Structural Identification Settings

StructuralIdentification = 'Choleski';           % Chose 'None' or 'Signs/Zeros' or 'Choleski';
agnostic = 'irfs';  % select: 'structural' or 'irfs';
       
    % Set up Sign Restrictions
        
      % SR{r} = {Shockname,{Variable Names}, Horizon, Sign (1 or -1),};
        
        SR{1} = {'MP',{'Nominal Rate'}                          ,0, 1};
        SR{2} = {'MP',{'Output Gap','Inflation'}                      ,0, -1};
        SR{3} = {'AD',{'Output Gap','Inflation','Nominal Rate'}                 ,0, -1};
        SR{4} = {'AS',{'Output Gap'}                           ,0, -1};
        SR{5} = {'AS',{'Inflation','Nominal Rate'}                     ,0, 1};
        
        SR{6} = {'MP',{'Nominal Rate'}                          ,1, 1};
        SR{7} = {'MP',{'Output Gap','Inflation'}                      ,1, -1};  
        SR{8} = {'AD',{'Output Gap','Inflation','Nominal Rate'}                 ,1, -1};
        SR{9} = {'AS',{'Output Gap'}                           ,1, -1};
        SR{10} = {'AS',{'Inflation','Nominal Rate'}                    ,1, 1};
       

cumulateWhich = []; % Compute Cumulated IRFs for Plots

    
%% Gibss Sampler Settings

numDesiredDraws = 5000;
BetaSigmaTries = 100;
Qs_per_BetaSigma = 100;

nRepsWeights = 1000;

% rng(1)

Run_SVAR_v1

%% Results

Plot_IRFs_supply


