
warning off;

options_.maxit_=1000000;

%% Define variables, parameters, and set values

var y pi i_nom e_d e_s e_i yminus1 piminus1 i_nomminus1 e_dminus1 e_sminus1 e_iminus1;
varexo eps_d eps_s eps_i;
parameters bet h mbar_c sig thet ome varphi rho_i phi_pi phi_y sig_d sig_s sig_i rho_d rho_s rho_e_i;

bet      = 0.99;
sig      = 1;
thet     = 0.66;
varphi   = 1;
rho_i    = 0.8;
phi_pi   = 1.50;
phi_y    = 0.6/4;
sig_d    = 0.4;
sig_s    = 0.4;
sig_i    = 0.4;
rho_d    = 0.8;
rho_s    = 0.8;
rho_e_i  = 0.8;
h        = 0.50;
ome      = 0.80;
mbar_c   = 0.46;
%set_param_value('mbar_c', mbar_c);


model(linear);
	
%% First, let's define some auxiliary expressions for the BDISC:

# lam_b = h/(1+h);
# lam_f = (1/(1+h))*mbar_c;
# lam_r = - (1-h)/(sig*(1+h));


%% Now, auxiliary expressions for the BNKPC:

# gam_b = ome/(1+ome*bet*mbar_c*(thet+(1-thet)*(1-thet*bet)/(1-thet*bet*mbar_c)));
# gam_mu = (((1-thet)*(1-thet*bet))/(thet*(1+ome*bet*mbar_c*(thet+(1-thet)*(1-thet*bet)/(1-thet*bet*mbar_c)))));
# gam_f = (bet*mbar_c*(thet+(1-thet)*(1-thet*bet)/(1-thet*bet*mbar_c))/(1+ome*bet*mbar_c*(thet+(1-thet)*(1-thet*bet)/(1-thet*bet*mbar_c))));
# alf_b = -(gam_mu*sig*h)/(1-h);
# alf_c = gam_mu*(varphi + (sig/(1-h)));

%%%% Finally the 3-Equation System:
   
%% BDISC:
y = lam_b*y(-1) + lam_f*y(+1) + lam_r*(i_nom - mbar_c*pi(+1)) + e_d;
e_d = rho_d*e_d(-1) + sig_d*eps_d;

%% BNKPC:
pi = gam_b*pi(-1) + alf_b*y(-1) + alf_c*y + gam_f*pi(+1) + e_s;
e_s = rho_s*e_s(-1) + sig_s*eps_s;

%% TR:
i_nom = rho_i*i_nom(-1) + (1-rho_i)*(phi_pi*pi + phi_y*y) + e_i;
e_i = rho_e_i*e_i(-1) + sig_i*eps_i;


%% Aux. eqs:

yminus1=y(-1);
piminus1=pi(-1);
i_nomminus1=i_nom(-1);

e_dminus1=e_d(-1);
e_sminus1=e_s(-1);
e_iminus1=e_i(-1);

end;


shocks;
    var eps_d; stderr 1;  
    var eps_s; stderr 1;
    var eps_i; stderr 1;    
end; 


set_dynare_seed(501);
stoch_simul(order=1,periods=10000,nograph,irf=0);


y_sim=y;
pi_sim=pi;
i_nom_sim=i_nom;

e_d_sim=e_d;
e_s_sim=e_s;
e_i_sim=e_i;

yminus1_sim=yminus1;
piminus1_sim=piminus1;
i_nomminus1_sim=i_nomminus1;

e_dminus1_sim=e_dminus1;
e_sminus1_sim=e_sminus1;
e_iminus1_sim=e_iminus1;



